#!/usr/bin/env nextflow

nextflow.enable.dsl=2
include { consolidateAnnotation;trainEuGene;buildEGNRepeatFile;launchAllEGN } from './modules/subworkflow/eugene.nf'
include { speciesSpecificRepeats;searchRepeat } from './modules/subworkflow/repeats.nf'
include { seqArbo;buildLargeOverlappingSequences;buildOverlappingSequences;processAdditionalFiles;estPredictions;proteinPredictions;abInitioPredictions;genomeIndex } from './modules/subworkflow/prepare.nf'
include { software_info } from './modules/subworkflow/software_info.nf'
include {EGNPCORE_getSoftMaskRegions} from './modules/software/egnpcore/main.nf'

workflow {
	main:

		if (params.help) {
			def help_message = NFcoreTemplate.help(workflow, params, log)
			log.info help_message
			exit 0
		}

		if(! params.output_prefix) { exit 1, 'output_prefix not specified' }
		if(! params.locus_tag_prefix) { exit 1, 'locus_tag_prefix not specified' }
		
		Utils.info("Reading configuration files", params.verbose)
		(params,configMap) = Utils.readParams(params);//,params.cfg) 
		Utils.info("Create config directory ${params.outdir}/01_cfg", params.verbose)
		File cfgDir = new File(params.outdir + "/01_cfg")
		cfgDir.mkdir()
		egn_params_only = Utils.dumpParams("${params.outdir}/01_cfg/run.parameters.cfg",params)
		run_parameters = Channel.fromPath(egn_params_only)
	
		if (params.prg_eugene_cfg_blastx) {Utils.assertFileExists('prg_eugene_cfg_blastx',params.prg_eugene_cfg_blastx)}
		if (params.prg_eugene_cfg_est) {Utils.assertFileExists('prg_eugene_cfg_est',params.prg_eugene_cfg_est)}
		if (params.prg_eugene_cfg_est_unspliced) {Utils.assertFileExists('prg_eugene_cfg_est_unspliced',params.prg_eugene_cfg_est_unspliced)}
		if (params.prg_eugene_cfg_ncrna_annotastruct) {Utils.assertFileExists('prg_eugene_cfg_blastx',params.prg_eugene_cfg_ncrna_annotastruct)}
		if (params.prg_eugene_euk_cfg_model) {Utils.assertFileExists('prg_eugene_euk_cfg_model',params.prg_eugene_euk_cfg_model)}
		
		if (params.sequences) { 

			(completeGenome,hardMaskedGenome,softMaskRegions) = EGNPCORE_getSoftMaskRegions(Channel.fromPath(params.sequences))
			
			softMaskRegionsTuples = Channel.empty()
			if ( params.soft_masked_genome ) {
				softMaskRegionsTuples = softMaskRegions.flatten().map{ n -> 
				seqid  = n.baseName.replaceAll("\\.softmask","")
				tuple(seqid, n)
				}
			}
			
			Utils.checkSequenceIds(params.sequences)
			Utils.info("Computing genome mode (normal or big)", params.verbose)
			configMap['genome_mode'] = Utils.getGenomeMode(params.sequences)
			Utils.info("Genome mode: " + configMap['genome_mode'], params.verbose)
			
			Utils.info("Computing genome hash", params.verbose)
			configMap['genome_hash'] = Utils.getFastaHash(params.sequences)
			Utils.info("Genome hash:" + configMap['genome_hash'], params.verbose)
			
			if (configMap['genome_size'] > 2**32) {
				Utils.info("Genome mode: big", params.verbose)
			}
			params['genome_size'] = configMap['genome_size']
			params['genome_hash'] = configMap['genome_hash']
		} else { exit 1, 'Input genome file not specified!' }
		
		if (params.mode == 'test') {
			pseudomolecules = seqArbo(completeGenome)
			overlappingSeq = buildOverlappingSequences(completeGenome)
		}
		else if (params.mode == 'abinitio') {
			pseudomolecules = seqArbo(completeGenome)
			overlappingSeq = buildOverlappingSequences(completeGenome)
			largeOverlappingSeq = buildLargeOverlappingSequences(completeGenome)
			(abInitioEvidences, maskingEvidences,ncRnaEvidences) = abInitioPredictions(completeGenome,pseudomolecules,overlappingSeq,largeOverlappingSeq)
		}
		else if (params.mode == 'protein') {
			overlappingSeq = buildOverlappingSequences(completeGenome)
			proteinEvidences = proteinPredictions(configMap,overlappingSeq)
		}
		else if (params.mode == 'est') {
			genomeIndexPath = genomeIndex(completeGenome,configMap.genome_mode,configMap.genome_hash)
			(estEvidences,rawGmap4Training,allEstConf,trainingSplitEstDBch) = estPredictions(configMap,configMap.genome_mode,configMap.genome_hash,completeGenome,genomeIndexPath)
		}
		else if (params.mode == 'train') {
			genomeIndexPath = genomeIndex(completeGenome,configMap.genome_mode,configMap.genome_hash)
			(estEvidences,rawGmap4Training,allEstConf,trainingSplitEstDBch) = estPredictions(configMap,configMap.genome_mode,configMap.genome_hash,completeGenome,genomeIndexPath)
			(matrix,noncan_cfg) = trainEuGene(completeGenome,configMap,configMap.genome_mode,configMap.genome_hash,rawGmap4Training,genomeIndexPath,trainingSplitEstDBch)
		}
		else if (params.mode == 'repeats') {
			pseudomolecules = seqArbo(completeGenome)
			overlappingSeq = buildOverlappingSequences(completeGenome)
			largeOverlappingSeq = buildLargeOverlappingSequences(completeGenome)
			(abInitioEvidences, maskingEvidences,ncRnaEvidences) = abInitioPredictions(completeGenome,pseudomolecules,overlappingSeq,largeOverlappingSeq)
			genomeIndexPath = genomeIndex(completeGenome,configMap.genome_mode,configMap.genome_hash)
			(estEvidences,rawGmap4Training,allEstConf,trainingSplitEstDBch) = estPredictions(configMap,configMap.genome_mode,configMap.genome_hash,completeGenome,genomeIndexPath)
			proteinEvidences = proteinPredictions(configMap,overlappingSeq)
			
			allAvailableEvidences = abInitioEvidences.concat(estEvidences,proteinEvidences)
			
			allEvidences = allAvailableEvidences.groupTuple().map{ t -> tuple(t.get(0),t.get(1).flatten())}
			(matrix,noncan_cfg) = trainEuGene(completeGenome,configMap,configMap.genome_mode,configMap.genome_hash,rawGmap4Training,genomeIndexPath,trainingSplitEstDBch)
			(repeatFasta, repeatFastaIndex) = speciesSpecificRepeats(completeGenome,pseudomolecules,configMap,matrix,noncan_cfg,allEvidences,allEstConf,run_parameters)
			repeatEvidences = searchRepeat(completeGenome, pseudomolecules,maskingEvidences,repeatFasta, repeatFastaIndex)
			okrepeatfile4egnTuples = buildEGNRepeatFile(configMap,pseudomolecules,repeatEvidences,ncRnaEvidences,estEvidences,proteinEvidences)
			
		}
		else {
			software_info()
			pseudomolecules = seqArbo(completeGenome)
			overlappingSeq = buildOverlappingSequences(completeGenome)
			largeOverlappingSeq = buildLargeOverlappingSequences(completeGenome)
			(abInitioEvidences, maskingEvidences,ncRnaEvidences) = abInitioPredictions(completeGenome,pseudomolecules,overlappingSeq,largeOverlappingSeq)

			if (params.soft_masked_genome) {
				Utils.info("Merge soft masked regions and predicted repeat regions", params.verbose)
				maskingEvidences = maskingEvidences.concat(softMaskRegionsTuples)
			}
			
			genomeIndexPath = genomeIndex(completeGenome,configMap.genome_mode,configMap.genome_hash)
			(estEvidences,rawGmap4Training,allEstConf,trainingSplitEstDBch) = estPredictions(configMap,configMap.genome_mode,configMap.genome_hash,completeGenome,genomeIndexPath)
			proteinEvidences = proteinPredictions(configMap,overlappingSeq)
			
			allAvailableEvidences = abInitioEvidences.concat(estEvidences,proteinEvidences)
			if (configMap.additional_list.size() > 0) {
				additionalEvidences = processAdditionalFiles(completeGenome,configMap)
				allAvailableEvidences = allAvailableEvidences.concat(additionalEvidences)
			}

			allEvidences = allAvailableEvidences.groupTuple().map{ t -> tuple(t.get(0),t.get(1).flatten())}
			(matrix,noncan_cfg) = trainEuGene(completeGenome,configMap,configMap.genome_mode,configMap.genome_hash,rawGmap4Training,genomeIndexPath,trainingSplitEstDBch)
			
			if (params.skip_repeat_masking){
				Utils.info("Skip repeat masking step", params.verbose)
				if (params.soft_masked_genome){
					//TODO repeatEvidences = soft mask positions
					Utils.info("Use soft masked regions as repeat regions", params.verbose)
					repeatEvidences = softMaskRegionsTuples
				}
				else {
					repeatEvidences = Channel.empty()
				}
			}
			else {
				(repeatFasta, repeatFastaIndex) = speciesSpecificRepeats(completeGenome,pseudomolecules,configMap,matrix,noncan_cfg,allEvidences,allEstConf,run_parameters)
				repeatEvidences = searchRepeat(completeGenome, pseudomolecules,maskingEvidences,repeatFasta, repeatFastaIndex)
			}
			okrepeatfile4egnTuples = buildEGNRepeatFile(configMap,pseudomolecules,repeatEvidences,ncRnaEvidences,estEvidences,proteinEvidences)
			finalEvidences = ncRnaEvidences.concat(estEvidences,proteinEvidences,repeatEvidences,okrepeatfile4egnTuples)
								.groupTuple().map{ 
											t -> tuple(t.get(0),t.get(1).flatten())
										}
			annotationV1gff3Tuples = launchAllEGN(pseudomolecules,matrix,noncan_cfg,finalEvidences,allEstConf,run_parameters,configMap.genome_mode)
			consolidateAnnotation(completeGenome, annotationV1gff3Tuples, ncRnaEvidences)
		}

		workflow.onComplete {
    		Utils.info("Pipeline completed.\nResults are located in ${params.outdir}", true)
    //println "Execution status: ${ workflow.success ? 'OK' : 'failed' }"
		}
		workflow.onError {
    		Utils.error("Oops... Pipeline execution stopped") //": ${workflow.errorMessage}"
		}
		
}

