#!/bin/bash


INFO='\033[0;32m'
NC='\033[0m'
ERROR='\033[0;31m'

ROOT_CONTAINER=https://lipm-browsers.toulouse.inra.fr/pub/singularity-repository/egnep/current/

SCRIPT_DIR="$( cd -- "$( dirname -- "${BASH_SOURCE[0]:-$0}"; )" &> /dev/null && realpath .. 2> /dev/null; )";
VERSION=`cat $SCRIPT_DIR/VERSION`

EGNPCORE_CONTAINER=egnpcore-$VERSION.sif
LIPMUTILS_CONTAINER=lipmutils-1.0.sif
USEARCH_CONTAINER=blast-2.12.0_usearch-v8.0.1623_i86linux64.sif
CDHIT_CONTAINER=cdhit-4.8.1.sif

LIST="plant fungi"

DFAM_VERSION="3.6"
DFAM_URL="https://www.dfam.org/releases/Dfam_$DFAM_VERSION/families/Dfam.embl.gz"
TRANSPOSONS_PSI_URL="https://raw.githubusercontent.com/NBISweden/TransposonPSI/master/transposon_ORF_lib/transposon_db.pep"
UNIPROT_URL="https://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/taxonomic_divisions/"

#convert_dfam_consensus.pl

USAGE=" --kingdom  <plant|fungi> --outdir <PATH TO OUTPUT DIRECTORY> --cpus <CPUS>"

if [ $# -lt 6 ]; then
  echo -e "\nUsage:\n$0 $USAGE\n"
  exit 0
fi


while [ $# -gt 0 ]; do

   if [[ $1 == *"--help"* ]]; then
        echo -e "\nUsage:\n$0 $USAGE\n"
        exit 0
   elif [[ $1 == *"--kingdom"* ]]; then
        KINGDOM=$2
   elif [[ $1 == *"--outdir"* ]]; then
        OUTDIR=`realpath $2`
   elif [[ $1 == *"--cpus"* ]]; then
        CPUS=$2
   fi

  shift
done

mkdir -p $OUTDIR

[[ $LIST =~ (^|[[:space:]])$KINGDOM($|[[:space:]]) ]] && OK=1 || OK=0

if [ $OK == 0 ]
then
	echo "${ERROR}ERROR - kingdom $KINGDOM not in '$LIST'${NC}"
	exit 1
fi

TREMBL_NAME=""
SPROT_NAME=""


if [ $KINGDOM == 'plant' ]; then
     TREMBL_NAME="uniprot_trembl_plants"
     SPROT_NAME="uniprot_sprot_plants"
elif [ $KINGDOM == 'fungi' ]; then
     TREMBL_NAME="uniprot_trembl_fungi"
     SPROT_NAME="uniprot_sprot_fungi"
fi


re='^[0-9]+$'
if ! [[ $CPUS =~ $re ]] ; then
   echo -e "${ERROR}ERROR - cpus >$CPUS< not valid - Integer required${NC}"
   exit 1
fi



if [ ! -e $EGNPCORE_CONTAINER ]; then
	echo -e "${INFO}[INFO] Retrieve EGNPCORE Container:  ${NC}\n\twget $ROOT_CONTAINER$EGNPCORE_CONTAINER\n"
	wget $ROOT_CONTAINER$EGNPCORE_CONTAINER
fi

if [ ! -e $LIPMUTILS_CONTAINER ]; then
	echo -e "${INFO}[INFO] Retrieve LipmUtils Container:  ${NC}\n\twget $ROOT_CONTAINER$LIPMUTILS_CONTAINER\n"
	wget $ROOT_CONTAINER$LIPMUTILS_CONTAINER
fi


if [ ! -e $USEARCH_CONTAINER ]; then
	echo -e "${INFO}[INFO] Retrieve Usearch Container:  ${NC}\n\twget $ROOT_CONTAINER/protected/$USEARCH_CONTAINER\n"
	wget $ROOT_CONTAINER/protected/$USEARCH_CONTAINER
fi

if [ ! -e $CDHIT_CONTAINER ]; then
	echo -e "${INFO}[INFO] Retrieve CD-Hit Container:  ${NC}\n\twget $ROOT_CONTAINER/$CDHIT_CONTAINER\n"
	wget $ROOT_CONTAINER/$CDHIT_CONTAINER
fi

chmod +x $EGNPCORE_CONTAINER
chmod +x $LIPMUTILS_CONTAINER
chmod +x $USEARCH_CONTAINER
chmod +x $CDHIT_CONTAINER


if [ ! -e $OUTDIR/Dfam$DFAM_VERSION.embl.gz ]; then
     echo -e "${INFO}[INFO] Retrieve Dfam $DFAM_VERSION file${NC}"
     wget -O $OUTDIR/Dfam$DFAM_VERSION.embl.gz $DFAM_URL
fi

if [ ! -e "$OUTDIR/Dfam-$DFAM_VERSION.consensus.fasta" ]; then
     echo -e "${INFO}[INFO] Convert Dfam $DFAM_VERSION file${NC}"
     singularity exec -B $OUTDIR $EGNPCORE_CONTAINER convert_dfam_consensus.pl --embl $OUTDIR/Dfam$DFAM_VERSION.embl.gz --outfasta $OUTDIR/Dfam-$DFAM_VERSION.consensus.fasta
fi


if [ ! -e $OUTDIR/transposon_db.pep ]; then
     echo -e "${INFO}[INFO] Retrieve TransposonDB file${NC}"
     wget -O $OUTDIR/transposon_db.pep $TRANSPOSONS_PSI_URL
fi

if [ ! -e "$OUTDIR/$SPROT_NAME.dat.gz" ]; then
     echo -e "${INFO}[INFO] Retrieve Swiss-Prot $SPROT_NAME file${NC}"
     wget -O $OUTDIR/$SPROT_NAME.dat.gz $UNIPROT_URL/$SPROT_NAME.dat.gz
fi

CMD="filter_uniprot_dat.pl  --dat $OUTDIR/$SPROT_NAME.dat.gz --pe 10 --min 50 --outfasta $OUTDIR/$SPROT_NAME.minlen50.fasta"
if [ ! -e "$OUTDIR/$SPROT_NAME.minlen50.fasta" ]; then
    echo -e "${INFO}[INFO] Convert Swiss-Prot $SPROT_NAME file${NC}"
    singularity exec -B $OUTDIR $EGNPCORE_CONTAINER $CMD
fi

if [ ! -e  "$OUTDIR/$SPROT_NAME.minlen50-cleaned_dfam.fasta" ];then
     echo -e "${INFO}[INFO] Clean Swiss-Prot $SPROT_NAME file with Dfam${NC}"
     singularity exec -B $OUTDIR $USEARCH_CONTAINER usearch -dbmask none -makeudb_usearch $OUTDIR/$SPROT_NAME.minlen50.fasta -output $OUTDIR/$SPROT_NAME.minlen50.fasta.udb
     singularity exec -B $OUTDIR $USEARCH_CONTAINER lipm_dbprot_remove_repbase.pl --blast_prg=usearch-blastx --in $OUTDIR/$SPROT_NAME.minlen50.fasta --out $OUTDIR/$SPROT_NAME.minlen50-cleaned_dfam.fasta --repbase $OUTDIR/Dfam-$DFAM_VERSION.consensus.fasta 
fi

if [ ! -e "$OUTDIR/$SPROT_NAME.minlen50-cleaned_dfam_transposons.fasta" ];then
     echo -e "${INFO}[INFO] Clean Swiss-Prot $SPROT_NAME file with TransposonDB${NC}"
     singularity exec -B $OUTDIR $USEARCH_CONTAINER usearch -dbmask none -makeudb_usearch $OUTDIR/transposon_db.pep -output $OUTDIR/transposon_db.pep.udb
     singularity exec -B $OUTDIR $USEARCH_CONTAINER lipm_dbprot_remove_repbase.pl --blast_prg=usearch-blastp --in $OUTDIR/$SPROT_NAME.minlen50-cleaned_dfam.fasta --out $OUTDIR/$SPROT_NAME.minlen50-cleaned_dfam_transposons.fasta --repbase $OUTDIR/transposon_db.pep
fi



if [ ! -e "$OUTDIR/$TREMBL_NAME.dat.gz" ]; then
    echo -e "${INFO}[INFO] Retrieve TrEMBL $TREMBL_NAME file${NC}"
    wget -O $OUTDIR/$TREMBL_NAME.dat.gz $UNIPROT_URL/$TREMBL_NAME.dat.gz
fi

CMD="filter_uniprot_dat.pl  --dat $OUTDIR/$TREMBL_NAME.dat.gz --pe 3 --min 50 --outfasta $OUTDIR/$TREMBL_NAME.pe3minlen50.fasta"
if [ ! -e "$OUTDIR/$TREMBL_NAME.pe3minlen50.fasta" ]; then
    echo -e "${INFO}[INFO] Convert TrEMBL $TREMBL_NAME file${NC}"
    singularity exec -B $OUTDIR $EGNPCORE_CONTAINER $CMD
fi

if [ ! -e  "$OUTDIR/$TREMBL_NAME.pe3minlen50-cleaned_dfam.fasta" ];then
     echo -e "${INFO}[INFO] Clean TrEMBL $TREMBL_NAME file with Dfam${NC}"
     singularity exec -B $OUTDIR $USEARCH_CONTAINER usearch -dbmask none -makeudb_usearch $OUTDIR/$TREMBL_NAME.pe3minlen50.fasta -output $OUTDIR/$TREMBL_NAME.pe3minlen50.fasta.udb
     singularity exec -B $OUTDIR $USEARCH_CONTAINER lipm_dbprot_remove_repbase.pl --blast_prg=usearch-blastx --in $OUTDIR/$TREMBL_NAME.pe3minlen50.fasta --out $OUTDIR/$TREMBL_NAME.pe3minlen50-cleaned_dfam.fasta --repbase $OUTDIR/Dfam-$DFAM_VERSION.consensus.fasta 
fi

if [ ! -e "$OUTDIR/$TREMBL_NAME.pe3minlen50-cleaned_dfam_transposons.fasta" ];then
     echo -e "${INFO}[INFO] Clean TrEMBL $TREMBL_NAME file with TransposonDB${NC}"
     singularity exec -B $OUTDIR $USEARCH_CONTAINER usearch -dbmask none -makeudb_usearch $OUTDIR/transposon_db.pep -output $OUTDIR/transposon_db.pep.udb
     singularity exec -B $OUTDIR $USEARCH_CONTAINER lipm_dbprot_remove_repbase.pl --blast_prg=usearch-blastp --in $OUTDIR/$TREMBL_NAME.pe3minlen50-cleaned_dfam.fasta --out $OUTDIR/$TREMBL_NAME.pe3minlen50-cleaned_dfam_transposons.fasta --repbase $OUTDIR/transposon_db.pep
fi

echo -e "${INFO}[INFO] Cluster TrEMBL $TREMBL_NAME file${NC}"
singularity exec -B $OUTDIR $CDHIT_CONTAINER cd-hit -M 1024000 -c 0.9 -s 0.8 -T $CPUS -i $OUTDIR/$TREMBL_NAME.pe3minlen50-cleaned_dfam_transposons.fasta -o  $OUTDIR/$TREMBL_NAME.pe3minlen50-cleaned_dfam_transposons.cdhit90.fasta


exit 0
