#!/bin/bash


INFO='\033[0;32m'
NC='\033[0m'
ERROR='\033[0;31m'

ROOT_CONTAINER=https://lipm-browsers.toulouse.inra.fr/pub/singularity-repository/egnep/current/
BARRNAP_CONTAINER=barrnap-0.9.sif
LIPMUTILS_CONTAINER=lipmutils-1.0.sif
KINGDOM=""
LIST="arc bac euk mito"
SEQUENCES=""
CPUS=2
USAGE=" --kingdom  <euk|arc|bac|mito> --sequences <PATH TO TRANSCRIPT FILE (fasta)> --cpus <CPUS>"

if [ $# -lt 6 ]; then
  echo -e "\nUsage:\n$0 $USAGE\n"
  exit 0
fi


while [ $# -gt 0 ]; do

   if [[ $1 == *"--help"* ]]; then
        echo -e "\nUsage:\n$0 $USAGE\n"
        exit 0
   elif [[ $1 == *"--kingdom"* ]]; then
        KINGDOM=$2
   elif [[ $1 == *"--sequences"* ]]; then
        SEQUENCES=$2
   elif [[ $1 == *"--cpus"* ]]; then
        CPUS=$2
   fi

  shift
done


[[ $LIST =~ (^|[[:space:]])$KINGDOM($|[[:space:]]) ]] && OK=1 || OK=0

if [ $OK == 0 ]
then
	echo -e "${ERROR}ERROR - kingdom $KINGDOM not in '$LIST'${NC}"
	exit 1
fi

if [ ! -f "$SEQUENCES" ]; then
	echo -e "${ERROR}ERROR - sequences ><not found${NC}"
	exit 1
fi

re='^[0-9]+$'
if ! [[ $CPUS =~ $re ]] ; then
   echo -e "${ERROR}ERROR - cpus >$CPUS< not valid - Integer required${NC}"
   exit 1
fi



if [ ! -e $BARRNAP_CONTAINER ]; then
	echo -e "${INFO}[INFO] Retrieve Barrnap Container:  ${NC}\n\twget $ROOT_CONTAINER$BARRNAP_CONTAINER\n"
	wget $ROOT_CONTAINER$BARRNAP_CONTAINER
fi

if [ ! -e $LIPMUTILS_CONTAINER ]; then
	echo -e "${INFO}[INFO] Retrieve LipmUtils Container:  ${NC}\n\twget $ROOT_CONTAINER$LIPMUTILS_CONTAINER\n"
	wget $ROOT_CONTAINER$LIPMUTILS_CONTAINER
fi

chmod +x $BARRNAP_CONTAINER
chmod +x $LIPMUTILS_CONTAINER




RAW_TRANSCRIPTS=`realpath $SEQUENCES`
THREADS=$CPUS
TRANSCRIPTS_DIR=`dirname $RAW_TRANSCRIPTS`

CMD=""
if [ ! -e  $RAW_TRANSCRIPTS.rrna-reads ]; then
	CMD="barrnap  -kingdom $KINGDOM --threads $THREADS $RAW_TRANSCRIPTS"
	echo -e "${INFO}[INFO] Run Barrnap:  ${NC}\n\tsingularity exec -B $TRANSCRIPTS_DIR ./$BARRNAP_CONTAINER $CMD\n"
	singularity exec -B $TRANSCRIPTS_DIR ./$BARRNAP_CONTAINER $CMD  > $RAW_TRANSCRIPTS.barrnap-$KINGDOM.gff3
	
	if [ $KINGDOM == 'euk' ];then
		CMD="barrnap  -kingdom mito --threads $THREADS $RAW_TRANSCRIPTS"
		echo -e "${INFO}[INFO] Run Barrnap in 'mito' mode:  ${NC}\n\tsingularity exec -B $TRANSCRIPTS_DIR ./$BARRNAP_CONTAINER $CMD\n"
		singularity exec -B $TRANSCRIPTS_DIR ./$BARRNAP_CONTAINER $CMD  >> $RAW_TRANSCRIPTS.barrnap-$KINGDOM.gff3
	fi

	cut -f1 $RAW_TRANSCRIPTS.barrnap-$KINGDOM.gff3 | grep -v '^#' | sort -u > $RAW_TRANSCRIPTS.rrna-reads
fi

CMD="lipm_fastafilter.pl --in $RAW_TRANSCRIPTS --remove --min_length 150 "
	
if [ -s $RAW_TRANSCRIPTS.rrna-reads ]; then
	RRNA_NUMBER=`wc -l $RAW_TRANSCRIPTS.rrna-reads`
	CMD="lipm_fastafilter.pl --in $RAW_TRANSCRIPTS --remove --min_length 150 --db_filename $RAW_TRANSCRIPTS.rrna-reads --db_column 1"
	echo -e "${INFO}[INFO] Remove $RRNA_NUMBER rRNA reads and short reads (min 150nt):  ${NC}\n\tsingularity exec -B $TRANSCRIPTS_DIR ./$LIPMUTILS_CONTAINER $CMD\n"
else
	echo -e "${INFO}[INFO] Remove short reads (min 150nt):  ${NC}\n\tsingularity exec -B $TRANSCRIPTS_DIR ./$LIPMUTILS_CONTAINER $CMD\n"
fi

singularity exec  -B $TRANSCRIPTS_DIR ./$LIPMUTILS_CONTAINER $CMD --out $RAW_TRANSCRIPTS.min150-norRNA.fasta

CMD="lipm_trim_polyX.pl --in  $RAW_TRANSCRIPTS.min150-norRNA.fasta --bothends --split-on-strech 15 --minlen 150"
echo -e "${INFO}[INFO] Trim PolyX:  ${NC}\n\tsingularity exec -B $TRANSCRIPTS_DIR ./$LIPMUTILS_CONTAINER $CMD\n"
singularity exec  -B $TRANSCRIPTS_DIR ./$LIPMUTILS_CONTAINER $CMD  --out $RAW_TRANSCRIPTS.min150-norRNA.trim_polyX-min150-bothends-splitstretch15.fasta	

CMD="lipm_nrdb -revcomp -o $RAW_TRANSCRIPTS.min150-norRNA.trim_polyX-min150-bothends-splitstretch15.nr.fasta	$RAW_TRANSCRIPTS.min150-norRNA.trim_polyX-min150-bothends-splitstretch15.fasta"
echo -e "${INFO}[INFO] Remove redundancy :  ${NC}\n\tsingularity exec -B $TRANSCRIPTS_DIR ./$LIPMUTILS_CONTAINER $CMD\n"
singularity exec  -B $TRANSCRIPTS_DIR ./$LIPMUTILS_CONTAINER $CMD

ln -s $RAW_TRANSCRIPTS.min150-norRNA.trim_polyX-min150-bothends-splitstretch15.fasta $RAW_TRANSCRIPTS.cleaned.fasta
echo -e "${INFO}[INFO] Cleaned file :  >>> $RAW_TRANSCRIPTS.cleaned.fasta\t <<< ${NC}"


exit 0